#include "ns3/applications-module.h"
#include "ns3/core-module.h"
#include "ns3/csma-module.h"
#include "ns3/internet-module.h"
#include "ns3/ipv4-global-routing-helper.h"
#include "ns3/network-module.h"
#include "ns3/point-to-point-module.h"
#include "ns3/netanim-module.h"
#include "ns3/mobility-helper.h"
#include "ns3/mobility-model.h"
using namespace ns3;
uint32_t nCsma = 10;

// Varibles to store the transmission and receiving time
double trans_time_1;
double rec_time_1;
double delay_1 = 0; // End-to-end Delay for transmission from node 1 to node 10
double trans_delay_1 = 0; // Transmission delay for transmission from node 1 to node 10
double throughput_1 = 0; // Throughput for transmission from node 1 to node 10

// Trace number of packets
uint32_t pkts = 10;
uint32_t pkt_cnt_1 = 0; // Packet count for transmission from node 1 to node 10
// Callback functions

static void
TxTraceClient(std::string context, Ptr<const Packet> p)
{
	// Use context to detect which node has transmitted the packet
	if (context[10] == '0') { // Node 1
		trans_time_1 = Simulator::Now().GetSeconds();
	} 
}

static void
RxTraceServer(std::string context,Ptr<const Packet> p)
{
  
    // Whenever a server receives packet, get the node ID using the context
    if (context[10] == '9') { // Node 10
    	rec_time_1 = Simulator::Now().GetSeconds();
    	delay_1 += (rec_time_1 - trans_time_1)*1000;
    	pkt_cnt_1++;
    }
    // If this the last packet, print the parameters
    if (pkt_cnt_1 == pkts) {
    	NS_LOG_UNCOND("Average end-to-end delay from node 1 to node 10: " << delay_1/pkts << " ms");
    	NS_LOG_UNCOND("Average throughput of transmission from node 1 to node 10: " << (p->GetSize()*pkts*8)/delay_1 << " bits/sec");
    		
    	// Increment pkt_cnt value to avoid printing the parameters again for transmission that completes early
    	pkt_cnt_1++;
    }

}

NS_LOG_COMPONENT_DEFINE("SecondScriptExample");

int
main(int argc, char* argv[])
{
    bool verbose = false;
    

    CommandLine cmd(__FILE__);
    cmd.AddValue("nCsma", "Number of \"extra\" CSMA nodes/devices", nCsma);
    cmd.AddValue("verbose", "Tell echo applications to log if true", verbose);

    cmd.Parse(argc, argv);

    if (verbose)
    {
        LogComponentEnable("UdpEchoClientApplication", LOG_LEVEL_INFO);
        LogComponentEnable("UdpEchoServerApplication", LOG_LEVEL_INFO);
    }

    nCsma = nCsma == 0 ? 1 : nCsma;
   
    NodeContainer csmaNodes;
    csmaNodes.Create(nCsma);

    CsmaHelper csma;
    csma.SetChannelAttribute("DataRate", StringValue("100Mbps"));
    csma.SetChannelAttribute("Delay", TimeValue(NanoSeconds(6560)));

    NetDeviceContainer csmaDevices;
    csmaDevices = csma.Install(csmaNodes);

    InternetStackHelper stack;
    stack.Install(csmaNodes);

    Ipv4AddressHelper address;

    address.SetBase("10.1.1.0", "255.255.255.0");
    Ipv4InterfaceContainer csmaInterfaces;
    csmaInterfaces = address.Assign(csmaDevices);

    UdpEchoServerHelper echoServerA(9); // Port 9 for server on Node 10	
    ApplicationContainer serverAppsA = echoServerA.Install(csmaNodes.Get(9));
    serverAppsA.Start(Seconds(1.0));
    serverAppsA.Stop(Seconds(10.0));
 
    UdpEchoClientHelper echoClientA(csmaInterfaces.GetAddress(9), 9); // Node 9 port 9
    echoClientA.SetAttribute("MaxPackets", UintegerValue(10));
    echoClientA.SetAttribute("Interval", TimeValue(Seconds(0.5)));
    echoClientA.SetAttribute("PacketSize", UintegerValue(1024));


    ApplicationContainer clientAppsA = echoClientA.Install(csmaNodes.Get(0));
    clientAppsA.Start(Seconds(2.0));
    clientAppsA.Stop(Seconds(10.0));


    Ipv4GlobalRoutingHelper::PopulateRoutingTables();

    
    // Trace connect for node 1 to node 10
    Config::Connect("/NodeList/0/ApplicationList/0/$ns3::UdpEchoClient/Tx", MakeCallback(&TxTraceClient));
    Config::Connect("/NodeList/9/ApplicationList/0/$ns3::UdpEchoServer/Rx", MakeCallback(&RxTraceServer));
   
    MobilityHelper mobility;
    mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel");
    mobility.Install(csmaNodes);
 
    // For netanim animation
    AnimationInterface anim ("csma-performance-analysis.xml");
	// Set node positions
	anim.SetConstantPosition(csmaNodes.Get(0),0,250);
	anim.SetConstantPosition(csmaNodes.Get(1),50,300);
	anim.SetConstantPosition(csmaNodes.Get(2),100,250);
	anim.SetConstantPosition(csmaNodes.Get(3),150,300);
	anim.SetConstantPosition(csmaNodes.Get(4),200,250);
	anim.SetConstantPosition(csmaNodes.Get(5),250,300);
	anim.SetConstantPosition(csmaNodes.Get(6),300,250);
	anim.SetConstantPosition(csmaNodes.Get(7),350,300);
	anim.SetConstantPosition(csmaNodes.Get(8),400,250);
	anim.SetConstantPosition(csmaNodes.Get(9),450,300);
    Simulator::Run();
    Simulator::Destroy();
    return 0;
}
